@@ -1,22 +1,64 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
-    anjuta-ui.c
-    Copyright (C) 2003  Naba Kumar  <naba@gnome.org>
+ * anjuta-ui.c
+ * Copyright (C) Naba Kumar  <naba@gnome.org>
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+/**
+ * SECTION:anjuta-ui
+ * @short_description: User Interface manager
+ * @see_also: #GtkAction, #GtkActionEntry, #GtkToggleAction,
+ *   #GtkToggleActionEntry, #GtkRadioAction, #GtkRadioActionEntry,
+ *   #GtkActionGroup, #GtkUIManager
+ * @stability: Unstable
+ * @include: libanjuta/
+ * 
+ * #AnjutaUI subclasses #GtkUIManager, so you should really read #GtkUIManager
+ * documentation first to know about Actions, UI merging and UI XML file
+ * format. This documentation will cover only the relevent	APIs.
+ * 
+ * #AnjutaUI has its own methods for adding action groups, which is differnt
+ * from #GtkUIManager methods. All #AnjutaPlugin based classes should use
+ * these methods instead of #GtkUIManager methods. The reason is, in addition
+ * to adding the actions and groups to the UI manager, it also resgisters
+ * them for UI customization and accellerators editing. It also keeps
+ * record of all actions.
+ * 
+ * An interesting side effect of this is that these
+ * actions could be conveniently accessed or activated with
+ * anjuta_ui_get_action() or anjuta_ui_activate_action_by_path(), without
+ * the need of original action group object. This makes it is possible for
+ * activating actions remotely from other plugins.
+ * 
+ * anjuta_ui_get_accel_editor() will return a widget containing the
+ * UI customization and accellerators editor. All actions and action groups
+ * are organized into a tree view, which should be added to a visible
+ * container (e.g. a #GtkDialog based object) and displayed to users.
+ * 
+ * <note>
+ * 	<para>
+ * 		Any actions additions/removals using #GtkUIManager are not
+ * 		registred with #AnjutaUI and hence their accellerators
+ * 		cannot be edited. Nor will they be listed in UI manager
+ * 		dialog. Hence, use #AnjutaUI methods whenever possible.
+ * 	</para>
+ * </note>
+ */
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -188,8 +230,8 @@ static gint
 iter_compare_func (GtkTreeModel *model, GtkTreeIter *a,
 				   GtkTreeIter *b, gpointer user_data)
 {
-	const gchar *text_a;
-	const gchar *text_b;
+	gchar *text_a;
+	gchar *text_b;
 	gint retval = 0;
 	
 	gtk_tree_model_get (model, a, COLUMN_ACTION, &text_a, -1);
@@ -198,6 +240,10 @@ iter_compare_func (GtkTreeModel *model, GtkTreeIter *a,
 	else if (text_a == NULL) retval = -1;
 	else if (text_b == NULL) retval = 1;
 	else retval =  strcasecmp (text_a, text_b);
+	
+	g_free(text_a);
+	g_free(text_b);
+	
 	return retval;
 }
 
@@ -771,6 +817,7 @@ anjuta_ui_merge (AnjutaUI *ui, const gchar *ui_filename)
 	{
 		gchar *basename = g_path_get_basename (ui_filename);
 		DEBUG_PRINT ("merged [%d] %s", id, basename);
+		g_free(basename);
 	}
 #endif
 	if (err != NULL)
@@ -789,7 +836,7 @@ anjuta_ui_merge (AnjutaUI *ui, const gchar *ui_filename)
 void
 anjuta_ui_unmerge (AnjutaUI *ui, gint id)
 {
-	DEBUG_PRINT ("Menu unmerging %d", id);
+	/* DEBUG_PRINT ("Menu unmerging %d", id); */
 	g_return_if_fail (ANJUTA_IS_UI (ui));
 	gtk_ui_manager_remove_ui(GTK_UI_MANAGER (ui), id);
 }
@@ -928,6 +975,6 @@ anjuta_ui_dump_tree (AnjutaUI *ui)
 	
 	gtk_ui_manager_ensure_update (GTK_UI_MANAGER (ui));
 	ui_str = gtk_ui_manager_get_ui (GTK_UI_MANAGER (ui));
-	DEBUG_PRINT ("%s", ui_str);
+	/* DEBUG_PRINT ("%s", ui_str); */
 	g_free (ui_str);
 }
