@@ -61,6 +61,7 @@ typedef struct _FileFilter
 	GList *file_unmatch;
 	GList *dir_match;
 	GList *dir_unmatch;
+	GList *ignore_pattern;
 	gboolean ignore_hidden_files;
 	gboolean ignore_hidden_dirs;
 	gboolean ignore_nonrepo_files;
@@ -294,6 +295,8 @@ anjuta_fv_open_file (FileManagerPlugin * fv, const char *path)
 #define DIR_FILTER_IGNORE_HIDDEN "filter.dir.ignore.hidden"
 #define FILE_FILTER_IGNORE_NONREPO "filter.file.ignore.nonrepo"
 
+#define DIR_FILTER_IGNORE_PATTERN "filter.file.ignore.pattern"
+
 #define GET_PREF(var, P) \
 	if (ff->var) \
 		anjuta_util_glist_strings_free(ff->var); \
@@ -320,6 +323,9 @@ fv_prefs_new (FileManagerPlugin *fv)
 	GET_PREF(dir_unmatch, DIR_FILTER_UNMATCH);
 	GET_PREF_BOOL(ignore_hidden_dirs, DIR_FILTER_IGNORE_HIDDEN);
 	GET_PREF_BOOL(ignore_nonrepo_files, FILE_FILTER_IGNORE_NONREPO);
+
+	GET_PREF(ignore_pattern, DIR_FILTER_IGNORE_PATTERN);
+
 	return ff;
 }
 
@@ -335,6 +341,8 @@ fv_prefs_free (FileFilter *ff)
 		anjuta_util_glist_strings_free (ff->dir_match);
 	if (ff->dir_unmatch)
 		anjuta_util_glist_strings_free (ff->dir_unmatch);
+	if (ff->ignore_pattern)
+		anjuta_util_glist_strings_free (ff->ignore_pattern);
 	g_free (ff);
 	ff = NULL;
 }
@@ -511,10 +519,12 @@ fv_add_tree_entry (FileManagerPlugin *fv, const gchar *path, GtkTreeIter *root)
 	struct dirent *dir_entry;
 	GtkTreeIter iter;
 	GdkPixbuf *pixbuf;
-	GSList *file_node;
-	GSList *files = NULL;
+	GList *file_node;
+	GList *files = NULL;
+	GList *directories = NULL;
 	GList *ignore_files = NULL;
 	gchar *entries = NULL;
+	GtkTreeViewColumn* column;
 
 	g_return_if_fail (path != NULL);
 
@@ -531,6 +541,7 @@ fv_add_tree_entry (FileManagerPlugin *fv, const gchar *path, GtkTreeIter *root)
 			{
 				g_free(entries);
 				entries = NULL;
+				
 			}
 			else
 			{
@@ -544,6 +555,9 @@ fv_add_tree_entry (FileManagerPlugin *fv, const gchar *path, GtkTreeIter *root)
 			}
 		}
 	}
+	column = gtk_tree_view_get_column(GTK_TREE_VIEW(fv->tree), 1);
+	gtk_tree_view_column_set_visible(column, entries != NULL);
+															 
 	
 	g_snprintf(file_name, PATH_MAX, "%s/.cvsignore", path);
 	if (ff->ignore_nonrepo_files && 0 == stat(file_name, &s))
@@ -572,7 +586,7 @@ fv_add_tree_entry (FileManagerPlugin *fv, const gchar *path, GtkTreeIter *root)
 				{
 					ignore_files = g_list_prepend (ignore_files,
 												   g_strdup (*ptrv));
-					DEBUG_PRINT ("Ignoring: %s", *ptrv);
+					/* DEBUG_PRINT ("Ignoring: %s", *ptrv); */
 					ptrv++;
 				}
 				g_strfreev (strv);
@@ -580,7 +594,20 @@ fv_add_tree_entry (FileManagerPlugin *fv, const gchar *path, GtkTreeIter *root)
 			}
 		}
 	}
-	
+
+	/* add ignore pattern */
+	{
+		GList *li;
+
+		li = ff->ignore_pattern;
+		while (li != NULL)
+		{
+			ignore_files = g_list_prepend (ignore_files, g_strdup (li->data));
+			DEBUG_PRINT ("Ignoring: %s", (gchar*)li->data);
+			li = g_list_next (li);
+		}
+	}
+
 	if (NULL != (dir = opendir(path)))
 	{
 		while (NULL != (dir_entry = readdir(dir)))
@@ -605,33 +632,46 @@ fv_add_tree_entry (FileManagerPlugin *fv, const gchar *path, GtkTreeIter *root)
 			
 			if (g_file_test (file_name, G_FILE_TEST_IS_DIR))
 			{
-				GtkTreeIter sub_iter;
-				/*if (!file_entry_apply_filter (file_name, ff->dir_match,
-											  ff->dir_unmatch,
-											  ff->ignore_hidden_dirs))
-					continue;
-				*/
-				pixbuf = gdl_icons_get_mime_icon (icon_set,
-											"application/directory-normal");
-				gtk_tree_store_append (store, &iter, root);
-				gtk_tree_store_set (store, &iter,
-							PIXBUF_COLUMN, pixbuf,
-							FILENAME_COLUMN, file,
-							REV_COLUMN, "D",
-							IS_DIR_COLUMN, 1,
-							-1);
-				g_object_unref (pixbuf);
-				gtk_tree_store_append (store, &sub_iter, &iter);
-				gtk_tree_store_set (store, &sub_iter,
-							FILENAME_COLUMN, _("Loading..."),
-							REV_COLUMN, "",
-							-1);
+				directories = g_list_prepend (directories, g_strdup (file));
 			} else {
-				files = g_slist_prepend (files, g_strdup (file));
+				files = g_list_prepend (files, g_strdup (file));
 			}
 		}
 		closedir(dir);
-		files = g_slist_reverse (files);
+		
+		/* Enter directories */
+		directories = anjuta_util_glist_strings_sort (directories);
+		file_node = directories;
+		while (file_node)
+		{
+			GtkTreeIter sub_iter;
+			gchar *dname = file_node->data;
+			/*if (!file_entry_apply_filter (file_name, ff->dir_match,
+			ff->dir_unmatch,
+			ff->ignore_hidden_dirs))
+			continue;
+			*/
+			pixbuf = gdl_icons_get_mime_icon (icon_set,
+											  "application/directory-normal");
+			gtk_tree_store_append (store, &iter, root);
+			gtk_tree_store_set (store, &iter,
+								PIXBUF_COLUMN, pixbuf,
+								FILENAME_COLUMN, dname,
+								REV_COLUMN, "",
+								IS_DIR_COLUMN, 1,
+								-1);
+			g_object_unref (pixbuf);
+			gtk_tree_store_append (store, &sub_iter, &iter);
+			gtk_tree_store_set (store, &sub_iter,
+								FILENAME_COLUMN, _("Loading..."),
+								REV_COLUMN, "",
+								-1);
+			g_free (dname);
+			file_node = g_list_next (file_node);
+		}
+		
+		/* Enter files */
+		files = anjuta_util_glist_strings_sort (files);
 		file_node = files;
 		while (file_node)
 		{
@@ -670,9 +710,10 @@ fv_add_tree_entry (FileManagerPlugin *fv, const gchar *path, GtkTreeIter *root)
 			gdk_pixbuf_unref (pixbuf);
 			g_free (version);
 			g_free (fname);
-			file_node = g_slist_next (file_node);
+			file_node = g_list_next (file_node);
 		}
-		g_slist_free (files);
+		g_list_free (files);
+		g_list_free (directories);
 	}
 	if (entries)
 		g_free (entries);
@@ -718,7 +759,7 @@ on_file_view_row_expanded (GtkTreeView *view,
 		}
 		while (gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &child));
 	}
-	
+
 	/* Update with new info */
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &child, iter_path);
 	full_path = fv_construct_full_path (fv, &child);
@@ -729,7 +770,7 @@ on_file_view_row_expanded (GtkTreeView *view,
 	pix = gdl_icons_get_mime_icon (icon_set, "application/directory-normal");
 	gtk_tree_store_set (store, &child, PIXBUF_COLUMN, pix, -1);
 	g_object_unref (pix);
-
+	
 	/* Delete the referenced rows */
 	row_ref_node = row_refs;
 	while (row_ref_node)
@@ -836,27 +877,6 @@ on_tree_view_selection_changed (GtkTreeSelection *sel, FileManagerPlugin *fv)
 	}
 }
 
-static gint
-compare_iter (GtkTreeModel *model, GtkTreeIter *iter1,
-			  GtkTreeIter *iter2, gpointer data)
-{
-	const gchar *filename1, *filename2;
-	gboolean is_dir1, is_dir2;
-	
-	gtk_tree_model_get (model, iter1, IS_DIR_COLUMN, &is_dir1, -1);
-	gtk_tree_model_get (model, iter2, IS_DIR_COLUMN, &is_dir2, -1);
-	if (is_dir1 && !is_dir2)
-		return -1;
-	else if (!is_dir1 && is_dir2)
-		return 1;
-	else
-	{
-		gtk_tree_model_get (model, iter1, FILENAME_COLUMN, &filename1, -1);
-		gtk_tree_model_get (model, iter2, FILENAME_COLUMN, &filename2, -1);
-		return g_ascii_strcasecmp (filename1, filename2);
-	}
-}
-
 void
 fv_init (FileManagerPlugin *fv)
 {
@@ -879,9 +899,6 @@ fv_init (FileManagerPlugin *fv)
 								G_TYPE_STRING,
 								G_TYPE_STRING,
 								G_TYPE_BOOLEAN);
-	gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (store),
-								compare_iter, fv, NULL);
-
 	fv->tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (fv->tree), FALSE);
 	
@@ -948,6 +965,7 @@ fv_finalize (FileManagerPlugin *fv)
 	
 	/* Object will be destroyed when removed from container */
 	/* gtk_widget_destroy (fv->scrolledwindow); */
+	fv_cancel_node_expansion (fv);
 	fv->top_dir = NULL;
 	fv->tree = NULL;
 	fv->scrolledwindow = NULL;
@@ -987,28 +1005,78 @@ fv_get_node_expansion_states (FileManagerPlugin *fv)
 	return map;
 }
 
+static gboolean
+on_fv_node_expansion_on_idle (gpointer user_data)
+{
+	GtkTreePath *path;
+	GtkTreeModel *model;
+	FileManagerPlugin *fv = ANJUTA_PLUGIN_FILE_MANAGER (user_data);
+	gchar *node_path = (gchar*)fv->nodes_to_expand->data;
+	
+	fv->nodes_to_expand = g_list_remove (fv->nodes_to_expand,
+											   node_path);
+	/* Expand node_path */
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (fv->tree));
+	path = gtk_tree_path_new_from_string (node_path);
+	DEBUG_PRINT ("Expanding node: %s", node_path);
+	gtk_tree_view_expand_row (GTK_TREE_VIEW (fv->tree), path, FALSE);
+	
+	/* Clean up */
+	gtk_tree_path_free (path);
+	g_free (node_path);
+	
+	/* End of this cycle */
+	if (fv->nodes_to_expand == NULL)
+	{
+		fv->idle_id = 0;
+		return FALSE; /* End */
+	}
+	else
+		return TRUE; /* Continue */
+}
+
+static void
+fv_queue_node_expansion (FileManagerPlugin *fv, const gchar *node_path)
+{
+	fv->nodes_to_expand = g_list_append (fv->nodes_to_expand,
+										 g_strdup (node_path));
+	if (fv->idle_id <= 0)
+		fv->idle_id = g_idle_add_full (G_PRIORITY_LOW,
+									   on_fv_node_expansion_on_idle,
+									   fv, NULL);
+}
+
 void
 fv_set_node_expansion_states (FileManagerPlugin *fv, GList *expansion_states)
 {
-	/* Restore expanded nodes */	
+	/* Queue expanded nodes */	
 	if (expansion_states)
 	{
-		GtkTreePath *path;
-		GtkTreeModel *model;
 		GList *node;
 		node = expansion_states;
 		
-		model = gtk_tree_view_get_model (GTK_TREE_VIEW (fv->tree));
 		while (node)
 		{
-			path = gtk_tree_path_new_from_string (node->data);
-			gtk_tree_view_expand_row (GTK_TREE_VIEW (fv->tree), path, FALSE);
-			gtk_tree_path_free (path);
+			fv_queue_node_expansion (fv, (gchar *)node->data);
 			node = g_list_next (node);
 		}
 	}
 }
 
+void
+fv_cancel_node_expansion (FileManagerPlugin *fv)
+{
+	if (fv->idle_id)
+		g_source_remove (fv->idle_id);
+	fv->idle_id = 0;
+	if (fv->nodes_to_expand)
+	{
+		g_list_foreach (fv->nodes_to_expand, (GFunc)g_free, NULL);
+		g_list_free (fv->nodes_to_expand);
+		fv->nodes_to_expand = NULL;
+	}
+}
+
 void
 fv_set_root (FileManagerPlugin *fv, const gchar *root_dir)
 {
@@ -1018,15 +1086,15 @@ fv_set_root (FileManagerPlugin *fv, const gchar *root_dir)
 		if (fv->top_dir)
 			g_free(fv->top_dir);
 		fv->top_dir = g_strdup(root_dir);
-		fv_refresh(fv);
+		fv_refresh(fv, FALSE);
 	}
 }
 
 void
-fv_refresh (FileManagerPlugin *fv)
+fv_refresh (FileManagerPlugin *fv, gboolean save_states)
 {
 	static gboolean busy = FALSE;
-	GList *selected_items;
+	GList *selected_items = NULL;
 	GtkTreeIter sub_iter;
 	GtkTreeIter iter;
 	GtkTreePath *path;
@@ -1034,12 +1102,16 @@ fv_refresh (FileManagerPlugin *fv)
 	GtkTreeStore *store;
 	GdkPixbuf *pixbuf;
 	gchar *project_dir;
-
+	gchar *root_node_path;
+	
 	if (busy)
 		return;
 	else
 		busy = TRUE;
 	
+	/* Make sure node expansion is stoped */
+	fv_cancel_node_expansion (fv);
+	
 	if (icon_set == NULL)
 		icon_set = gdl_icons_new (16);
 	if (ff != NULL)
@@ -1047,7 +1119,10 @@ fv_refresh (FileManagerPlugin *fv)
 	ff = fv_prefs_new (fv);
 	
 	fv_disconnect (fv);
-	selected_items = fv_get_node_expansion_states (fv);
+
+	if (save_states)
+		selected_items = fv_get_node_expansion_states (fv);
+
 	fv_clear (fv);
 
 	project_dir = g_path_get_basename (fv->top_dir);
@@ -1071,18 +1146,20 @@ fv_refresh (FileManagerPlugin *fv)
 				FILENAME_COLUMN, _("Loading..."),
 				REV_COLUMN, "",
 				-1);
-
-	/* Expand first node */
-	gtk_tree_model_get_iter_first (model, &iter);
-	path = gtk_tree_model_get_path (model, &iter);
-	gtk_tree_view_expand_row (GTK_TREE_VIEW (fv->tree), path, FALSE);
-	gtk_tree_path_free (path);
-
-	fv_set_node_expansion_states (fv, selected_items);
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
-										  GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
-										  GTK_SORT_ASCENDING);
-
+	if (save_states)
+	{
+		fv_set_node_expansion_states (fv, selected_items);
+	}
+	else
+	{
+		/* Expand first node */
+		gtk_tree_model_get_iter_first (model, &iter);
+		path = gtk_tree_model_get_path (model, &iter);
+		root_node_path = gtk_tree_path_to_string (path);
+		fv_queue_node_expansion (fv, root_node_path);
+		gtk_tree_path_free (path);
+		g_free (root_node_path);
+	}
 	if (selected_items)
 		anjuta_util_glist_strings_free (selected_items);
 	fv_connect (fv);
