@@ -27,7 +27,7 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: anjuta-document.c,v 1.9 2006/05/13 13:54:32 jhs Exp $
+ * $Id: anjuta-document.c 3224 2007-10-17 12:59:15Z jhs $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -58,10 +58,6 @@
 
 static void	anjuta_document_set_readonly	(AnjutaDocument *doc,
 						 gboolean       readonly);
-static void insert_text_cb		 	(AnjutaDocument *doc, 
-						 GtkTextIter   *pos,
-						 const gchar   *text, 
-						 gint           length);
 						 
 static void	delete_range_cb 		(AnjutaDocument *doc, 
 						 GtkTextIter   *start,
@@ -73,7 +69,6 @@ struct _AnjutaDocumentPrivate
 	gint	     last_save_was_manually : 1; 	
 	gint	     language_set_by_user : 1;
 	gint         is_saving_as : 1;
-	gint         has_selection : 1;
 	gint         stop_cursor_moved_emission : 1;
 
 	gchar	    *uri;
@@ -107,8 +102,7 @@ enum {
 	PROP_SHORTNAME,
 	PROP_MIME_TYPE,
 	PROP_READ_ONLY,
-	PROP_ENCODING,
-	PROP_HAS_SELECTION,
+	PROP_ENCODING
 };
 
 enum {
@@ -117,6 +111,7 @@ enum {
 	LOADED,
 	SAVING,
 	SAVED,
+	CHAR_ADDED,
 	LAST_SIGNAL
 };
 
@@ -184,7 +179,7 @@ anjuta_document_finalize (GObject *object)
 	if (doc->priv->uri != NULL)
 	{
 		GtkTextIter iter;
-		gchar *position;
+		// gchar *position;
 		gchar *lang_id = NULL;
 		GtkSourceLanguage *lang;
 
@@ -239,9 +234,6 @@ anjuta_document_get_property (GObject    *object,
 		case PROP_ENCODING:
 			g_value_set_boxed (value, doc->priv->encoding);
 			break;
-		case PROP_HAS_SELECTION:
-			g_value_set_boolean (value, doc->priv->has_selection);
-			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -254,7 +246,7 @@ anjuta_document_set_property (GObject      *object,
 			     const GValue *value,
 			     GParamSpec   *pspec)
 {
-	AnjutaDocument *doc = ANJUTA_DOCUMENT (object);
+	// AnjutaDocument *doc = ANJUTA_DOCUMENT (object);
 
 	switch (prop_id)
 	{
@@ -287,22 +279,6 @@ anjuta_document_mark_set (GtkTextBuffer     *buffer,
 									       iter,
 									       mark);
 
-	if (mark == gtk_text_buffer_get_insert (buffer) ||
-	    mark == gtk_text_buffer_get_selection_bound (buffer))
-	{
-		gboolean has_selection;
-
-		has_selection = gtk_text_buffer_get_selection_bounds (buffer,
-								      NULL,
-								      NULL);
-
-		if (has_selection != doc->priv->has_selection)
-		{
-			doc->priv->has_selection = has_selection;
-			g_object_notify (G_OBJECT (doc), "has-selection");
-		}
-	}
-
 	if (mark == gtk_text_buffer_get_insert (buffer))
 	{
 		emit_cursor_moved (doc);
@@ -329,7 +305,7 @@ anjuta_document_class_init (AnjutaDocumentClass *klass)
 
 	buf_class->mark_set = anjuta_document_mark_set;
 	buf_class->changed = anjuta_document_changed;
-
+	
 	g_object_class_install_property (object_class, PROP_URI,
 					 g_param_spec_string ("uri",
 					 		      "URI",
@@ -361,16 +337,6 @@ anjuta_document_class_init (AnjutaDocumentClass *klass)
 					 		     ANJUTA_TYPE_ENCODING,
 					 		     G_PARAM_READABLE));
 
-	/* This has been properly moved in GtkTextBuffer in gtk 2.10, so when
-	 * we switch to 2.10 we can remove it and part of with anjuta_document_mark_set.
-	 */
-	g_object_class_install_property (object_class, PROP_HAS_SELECTION,
-					 g_param_spec_boolean ("has-selection",
-					 		       "Has selection",
-					 		       "Wheter the document has selected text",
-					 		       FALSE,
-					 		       G_PARAM_READABLE));
-
 	/* This signal is used to update the cursor position is the statusbar,
 	 * it's emitted either when the insert mark is moved explicitely or
 	 * when the buffer changes (insert/delete).
@@ -432,7 +398,7 @@ anjuta_document_class_init (AnjutaDocumentClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_POINTER);
-
+				  
 	g_type_class_add_private (object_class, sizeof(AnjutaDocumentPrivate));
 }
 
@@ -509,7 +475,6 @@ anjuta_document_init (AnjutaDocument *doc)
 
 	doc->priv->readonly = FALSE;
 
-	doc->priv->has_selection = FALSE;
 	doc->priv->stop_cursor_moved_emission = FALSE;
 
 	doc->priv->last_save_was_manually = TRUE;
@@ -524,11 +489,6 @@ anjuta_document_init (AnjutaDocument *doc)
 	gtk_source_buffer_set_check_brackets (GTK_SOURCE_BUFFER (doc), 
 					     TRUE);
 
-	g_signal_connect_after (doc, 
-			  	"insert-text",
-			  	G_CALLBACK (insert_text_cb),
-			  	NULL);
-
 	g_signal_connect_after (doc, 
 			  	"delete-range",
 			  	G_CALLBACK (delete_range_cb),
@@ -607,7 +567,7 @@ set_uri (AnjutaDocument *doc,
 
 	if (!doc->priv->language_set_by_user)
 	{
-		gchar *data;
+		// gchar *data;
 		GtkSourceLanguage *language = NULL;
 
 		
@@ -949,15 +909,15 @@ document_saver_saving (AnjutaDocumentSaver *saver,
 				      TRUE);
 		}
 
-		g_signal_emit (doc,
-			       document_signals[SAVED],
-			       0,
-			       error);
-
 		/* the saver has been used, throw it away */
 		g_object_unref (doc->priv->saver);
 		doc->priv->saver = NULL;
 		doc->priv->is_saving_as = FALSE;
+		
+		g_signal_emit (doc,
+			       document_signals[SAVED],
+			       0,
+			       error);
 	}
 	else
 	{
@@ -1066,14 +1026,6 @@ anjuta_document_get_deleted (AnjutaDocument *doc)
 	return !gnome_vfs_uri_exists (doc->priv->vfs_uri);
 }
 
-gboolean
-_anjuta_document_get_has_selection (AnjutaDocument *doc)
-{
-	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (doc), FALSE);
-
-	return doc->priv->has_selection;
-}
-
 /*
  * If @line is bigger than the lines of the document, the cursor is moved
  * to the last line and FALSE is returned.
@@ -1134,64 +1086,62 @@ anjuta_document_get_encoding (AnjutaDocument *doc)
 	return doc->priv->encoding;
 }
 
-#define WORD_REGEX "[^ \\t&*!(]+$"
-gchar* anjuta_document_get_current_word(AnjutaDocument* doc)
+static gboolean
+wordcharacters_contains (gchar c)
 {
-	pcre *re;
-    gint err_offset;
-	const gchar* err;
-	gint rc, start, end;
-	int ovector[2];
-	gchar* line, *word;
-	GtkTextIter *line_iter, cursor_iter;
-	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(doc);
-	gtk_text_buffer_get_iter_at_mark(buffer, &cursor_iter, 
-								 gtk_text_buffer_get_insert(buffer));
+	const gchar* wordcharacters =
+		"_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+	gint pos;
 	
-	line_iter = gtk_text_iter_copy(&cursor_iter);
-	gtk_text_iter_set_line_offset(line_iter, 0);
-	line = gtk_text_buffer_get_text(buffer, line_iter, &cursor_iter, FALSE);
-	gtk_text_iter_free(line_iter);
-
-	/* Create regular expression */
-	re = pcre_compile(WORD_REGEX, 0, &err, &err_offset, NULL);
-	
-	if (NULL == re)
+	for (pos = 0; pos < strlen(wordcharacters); pos++)
 	{
-		/* Compile failed - check error message */
-		DEBUG_PRINT("Regex compile failed! %s at position %d", err, err_offset);
-		return FALSE;
+		if (wordcharacters[pos] == c)
+		{
+			return TRUE;
+		}
 	}
-	
-	/* Run the next matching operation */
-	rc = pcre_exec(re, NULL, line, strlen(line), 0,
-					   0, ovector, 2);
+	return FALSE;
+}
 
-	 if (rc == PCRE_ERROR_NOMATCH)
-    {
-    	DEBUG_PRINT("No match");
-    	g_free(line);
-    	return NULL;
-    }
- 	 else if (rc < 0)
-    {
-    	DEBUG_PRINT("Matching error %d\n", rc);
-    	return NULL;
-    }
-	else if (rc == 0)
-    {
-    	DEBUG_PRINT("ovector too small");
-    	return NULL;
-    }
-    start = ovector[0];
-    end = ovector[1];
-    	    	
-    word = g_new0(gchar, end - start + 1);
-    strncpy(word, line + start, end - start);
-  	
-  	g_free(line);
-  	
-  	return word;
+gchar* anjuta_document_get_current_word(AnjutaDocument* doc, gboolean end_position)
+{
+	GtkTextIter begin;
+	GtkTextIter end;
+	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(doc);
+	gchar* region;
+	gchar* word;
+	gint startword;
+	gint endword;
+	int cplen;
+	const int maxlength = 100;
+	
+	gtk_text_buffer_get_iter_at_mark (buffer, &begin, 
+																		gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(buffer)));
+	gtk_text_buffer_get_iter_at_mark (buffer, &end, 
+																		gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(buffer)));
+	startword = gtk_text_iter_get_line_offset (&begin);	
+	endword = gtk_text_iter_get_line_offset (&end);
+	
+	gtk_text_iter_set_line_offset (&begin, 0);
+	gtk_text_iter_forward_to_line_end (&end);
+	
+	region = gtk_text_buffer_get_text (buffer, &begin, &end, FALSE);
+	
+	while (startword> 0 && wordcharacters_contains(region[startword - 1]))
+		startword--;
+	while (!end_position && region[endword] && wordcharacters_contains(region[endword]))
+		endword++;
+	if(startword == endword)
+		return NULL;
+	
+	region[endword] = '\0';
+	cplen = (maxlength < (endword-startword+1))?maxlength:(endword-startword+1);
+	word = g_strndup (region + startword, cplen);
+	
+	DEBUG_PRINT ("region: %s\n start: %d end: %d word: %s", region, startword, endword, word);
+	g_free(region);
+	
+	return word;
 }
 
 glong
@@ -1213,27 +1163,6 @@ _anjuta_document_is_saving_as (AnjutaDocument *doc)
 	
 	return (doc->priv->is_saving_as);
 }
-
-static void
-insert_text_cb (AnjutaDocument *doc, 
-		GtkTextIter   *pos,
-		const gchar   *text, 
-		gint           length)
-{
-	GtkTextIter start;
-	GtkTextIter end;
-		
-	start = end = *pos;
-
-	/*
-	 * pos is invalidated when
-	 * insertion occurs (because the buffer contents change), but the
-	 * default signal handler revalidates it to point to the end of the
-	 * inserted text 
-	 */
-	gtk_text_iter_backward_chars (&start,
-				      g_utf8_strlen (text, length));
-}
 						 
 static void	
 delete_range_cb (AnjutaDocument *doc, 
@@ -1247,3 +1176,4 @@ delete_range_cb (AnjutaDocument *doc,
 	d_end = *end;
 	
 }
+
